REM ############################################################################################
REM # MSWIN platform - Oracle Database Logical Backup Script via Data Pump 
REM # Author: Amos Geng
REM # Date: 01/15/2018
REM # Description: Backup Oracle database via utility Data Pump 
REM # Action: Oracle directory DATA_PUMP_BACKUP needs to be created within database in advanced
REM ############################################################################################

@echo on

goto main

:funct_set_variable
set var_database_name=<DB_NAME>
set var_hostname=%COMPUTERNAME%
set var_filedatetime=%DATE:~10,4%_%DATE:~4,2%_%DATE:~7,2%_%TIME:~0,2%%TIME:~3,2%%TIME:~6,2%
set var_filedatetime=%var_filedatetime: =%
set var_script_name=emeralit_mswin_expdp_backup
set var_script_location=c:\emeralit\scripts
set var_script_log_location=c:\emeralit\logs
set var_dump_directory=E:\db_export
set var_dump_retention=3
set var_client_name=Emeralit
set var_7zip_password=<password_for_unzipping_compressed_file>
set var_notification_email=support@emeralit.com
goto:EOF



:funct_export
set oracle_sid=%var_database_name%
expdp \"/ as sysdba\" full=Y directory=DATA_PUMP_BACKUP dumpfile=expdp_%ORACLE_SID%_%var_filedatetime%.dmp logfile=expdp_%ORACLE_SID%_%var_filedatetime%.log
goto:EOF


:funct_log_cleanup
forfiles /P %var_dump_directory%      /M expdp_*.log /D -%var_dump_retention% /C "cmd /c Del @FILE"
forfiles /P %var_dump_directory%      /M expdp_*.dmp /D -%var_dump_retention% /C "cmd /c Del @FILE"
forfiles /P %var_dump_directory%      /M expdp_*.7z  /D -%var_dump_retention% /C "cmd /c Del @FILE"
forfiles /P %var_script_log_location% /M aws_dmp_shipment_*.txt /D -%var_dump_retention% /C "cmd /c Del @FILE"
forfiles /P %var_script_log_location% /M expdp_backup_notification*.txt /D -%var_dump_retention% /C "cmd /c Del @FILE"
goto:EOF



:funct_oraerr_notice_zip
set var_errors_status=no_error
for /f "delims=" %%i in ('findstr /n "ORA-" %var_dump_directory%\expdp_%ORACLE_SID%_%var_filedatetime%.log') do set var_errors_status=detected

if %var_errors_status% == detected (
     echo to: %var_notification_email%                                                               > %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo from: support@emeralit.com                                                                >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo subject: Proactive - %var_client_name%:%var_hostname%:Expdp Backup Errors Detected        >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo.>>                                                                                           %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     type %var_dump_directory%\expdp_%ORACLE_SID%_%var_filedatetime%.log                            >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo.>>                                                                                           %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo Notice sent by %var_script_location%\%var_script_name%                                    >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     Msmtp -t <                                                                                        %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
) else (
	 REM zip -9 -m -o %var_dump_directory%\expdp_%ORACLE_SID%_%var_filedatetime%.dmp.zip %var_dump_directory%\expdp_%ORACLE_SID%_%var_filedatetime%.dmp
	 C:\"Program Files"\7-Zip\7z.exe a -t7z -mx=9 -p%var_7zip_password% %var_dump_directory%\expdp_%ORACLE_SID%_%var_filedatetime%.dmp.7z %var_dump_directory%\expdp_%ORACLE_SID%_%var_filedatetime%.dmp > %var_script_log_location%\expdp_7zip_%ORACLE_SID%_%var_filedatetime%.txt

     aws s3 cp e:\db_export\expdp_%ORACLE_SID%_%var_filedatetime%.dmp.7z s3://aws-db-backup-emeralit > %var_script_log_location%\aws_dmp_shipment_%ORACLE_SID%_%var_filedatetime%.txt
	 
	 echo to: %var_notification_email%                                                               > %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo from: support@emeralit.com                                                                >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo subject: Informative - %var_client_name%:%var_hostname%:Expdp Completed                   >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo.>>                                                                                           %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
	 type %var_dump_directory%\expdp_%ORACLE_SID%_%var_filedatetime%.log                            >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
	 echo.>>                                                                                           %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
	 echo.>>                                                                                           %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
	 type %var_script_log_location%\expdp_7zip_%ORACLE_SID%_%var_filedatetime%.txt                  >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
	 echo.>>                                                                                           %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     type %var_script_log_location%\aws_dmp_shipment_%ORACLE_SID%_%var_filedatetime%.txt            >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo.>>                                                                                           %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     echo Notice sent by %var_script_location%\%var_script_name%                                    >> %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
     Msmtp -t <                                                                                        %var_script_log_location%\expdp_backup_notification_%ORACLE_SID%_%var_filedatetime%.txt
)
goto:EOF



:main
call :funct_set_variable
call :funct_export
call :funct_oraerr_notice_zip
call :funct_log_cleanup
goto:EOF